from sawsi.aws import shared
import boto3
import boto3.exceptions
from typing import Any


class LambdaAPI:
    def __init__(self, credentials=None, region=shared.DEFAULT_REGION):
        self.boto3_session = shared.get_boto_session(credentials)
        self.lambda_client = self.boto3_session.client('lambda', region_name=region)

    def invoke(self, function_name: str, payload: Any):
        response = self.lambda_client.invoke(
            FunctionName=function_name,
            InvocationType='RequestResponse',  # 'Event' for asynchronous execution
            Payload=payload
        )

        # 응답에서 Payload 추출
        response_payload = response['Payload'].read()
        response_body = response_payload.decode('utf-8')
        return response_body

    def create_event_source_mapping(
            self, function_name: str, event_source_arn: str,
            enabled: bool = True, batch_size: int = 100, starting_position: str = 'LATEST'
    ):
        """
        Lambda 함수에 트리거 추가
        """
        response = self.lambda_client.create_event_source_mapping(
            EventSourceArn=event_source_arn,
            FunctionName=function_name,
            Enabled=enabled,
            BatchSize=batch_size,  # 처리할 항목 수 조정 가능
            StartingPosition=starting_position  # 최신 항목부터 시작
        )
        return response


    def update_lambda_code(self, function_name, s3_bucket, s3_key, publish=False):
        """
        Updates the code of an existing Lambda function.

        :param function_name: Name of the Lambda function to update.
        :param s3_bucket: S3 bucket where the updated Lambda code is stored.
        :param s3_key: Key of the S3 object containing the updated Lambda code.
        :param publish: Whether to publish a new version of the function.
        :return: Response from the Lambda update_function_code API call.
        """
        try:
            response = self.lambda_client.update_function_code(
                FunctionName=function_name,
                S3Bucket=s3_bucket,
                S3Key=s3_key,
                Publish=publish
            )
            print("Function code updated successfully.")
            return response
        except boto3.exceptions.Boto3Error as e:
            print(f"Error updating Lambda function code: {e}")
            raise

    def update_lambda_configuration(self, function_name, memory_size=None, timeout=None):
        """
        Updates the configuration of an existing Lambda function.

        :param function_name: Name of the Lambda function to update.
        :param memory_size: (Optional) Memory size for the Lambda function in MB.
        :param timeout: (Optional) Timeout for the Lambda function in seconds.
        :return: Response from the Lambda update_function_configuration API call.
        """
        try:
            if not memory_size and not timeout:
                print("No configuration parameters provided for update.")
                return

            config_params = {}
            if memory_size:
                config_params['MemorySize'] = memory_size
            if timeout:
                config_params['Timeout'] = timeout

            response = self.lambda_client.update_function_configuration(
                FunctionName=function_name,
                **config_params
            )
            print("Function configuration updated successfully.")
            return response
        except boto3.exceptions.Boto3Error as e:
            print(f"Error updating Lambda function configuration: {e}")
            raise


    def create_lambda_function(self, function_name, role_arn, s3_bucket, s3_key, handler, runtime="python3.13"):
        """
        Creates a new AWS Lambda function.

        :param function_name: Name of the new Lambda function.
        :param role_arn: ARN of the IAM role for the function.
        :param s3_bucket: S3 bucket containing the function code.
        :param s3_key: Key of the S3 object with the code.
        :param handler: Function handler (e.g., "app.handler").
        :param runtime: Runtime environment (default: "python3.9").
        :return: Response from the create_function API call.
        """
        try:
            response = self.lambda_client.create_function(
                FunctionName=function_name,
                Runtime=runtime,
                Role=role_arn,
                Handler=handler,
                Code={
                    'S3Bucket': s3_bucket,
                    'S3Key': s3_key
                },
                Publish=True
            )
            return response
        except boto3.exceptions.Boto3Error as e:
            print(f"Error creating Lambda function: {e}")
            raise


    def delete_lambda_function(self, function_name):
        """
        Deletes an existing AWS Lambda function.

        :param function_name: Name of the Lambda function to delete.
        :return: Response from the delete_function API call.
        """
        try:
            response = self.lambda_client.delete_function(FunctionName=function_name)
            return response
        except boto3.exceptions.Boto3Error as e:
            print(f"Error deleting Lambda function: {e}")
            raise

    def get_lambda_function_info(self, function_name):
        """
        Retrieves information about an existing AWS Lambda function.

        :param function_name: Name of the Lambda function.
        :return: Response from the get_function API call.
        """
        try:
            response = self.lambda_client.get_function(FunctionName=function_name)
            return response
        except boto3.exceptions.Boto3Error as e:
            print(f"Error retrieving Lambda function info: {e}")
            raise

    def update_lambda_environment_variables(self, function_name, environment_variables):
        """
        Updates the environment variables of a Lambda function.

        :param function_name: Name of the Lambda function.
        :param environment_variables: Dictionary of environment variables to update.
        :return: Response from the update_function_configuration API call.
        """
        try:
            response = self.lambda_client.update_function_configuration(
                FunctionName=function_name,
                Environment={
                    'Variables': environment_variables
                }
            )
            return response
        except boto3.exceptions.Boto3Error as e:
            print(f"Error updating Lambda environment variables: {e}")
            raise


from sawsi.shared import handler_util
from sawsi.shared.schedule_util import ScheduledTask, join

# X-ray 추적


# 아래 핸들러는 share.error_util.AppError 에러가 발생할시에, 자동으로
# 에러를 response 객체에 담아 코드와 메시지로 구분하여 전송합니다.
@handler_util.aws_handler_wrapper(
    error_receiver=lambda errmsg: print(errmsg),  # 이 Lambda 함수를 슬랙 Webhook 등으로 대체하면 에러 발생시 모니터링이 가능합니다.
    content_type='application/json',  # 기본적으로 JSON 타입을 반환합니다.
    use_traceback=True,  # 에러 발생시 상세 값을 응답에 전달할지 유무입니다.
)
def handler(event, context):
    """
    AWS CloudWatch Event Bridge - Scheduler 를 통해 트리거됨
    중요: 일정을 (Rate 1 minute) 으로 설정시 자동으로 아래 함수들이 맞는 일정에 맞춰 실행 됨.
    """
    # 매일 0시 0분에 수행
    with ScheduledTask('0 0 * * *') as do:
        def task_to_do():
            # SHOULD IMP
            # TASK ...
            pass

        do(task_to_do)

    # 15분 마다 수행
    with ScheduledTask('*/15 * * * *') as do:
        def task_to_do_1():
            # TASK ...
            # SHOULD IMP
            pass

        def task_to_do_2():
            # TASK ...
            # SHOULD IMP
            pass

        do(task_to_do_1)
        do(task_to_do_2)

    # Join until task is done
    join()


import os
import subprocess
import platform


def install_aws_es_proxy():
    """
    Detect the operating system and install aws-es-proxy along with its dependencies.
    This function will install Homebrew (if needed), Go, and aws-es-proxy based on the OS.
    It also sets up the required environment variables.
    """
    os_name = platform.system()

    if os_name == "Darwin":  # macOS
        install_brew()  # Install Homebrew if not installed
        install_go_mac()  # Install Go on macOS using Homebrew
        set_environment_variable_mac_linux()  # Set environment variables
        install_aws_es_proxy_command()  # Install aws-es-proxy
    elif os_name == "Linux":
        install_go_linux()  # Install Go on Linux
        set_environment_variable_mac_linux()  # Set environment variables
        install_aws_es_proxy_command()  # Install aws-es-proxy
    elif os_name == "Windows":
        install_go_windows()  # Install Go on Windows
        set_environment_variable_windows()  # Set environment variables
        install_aws_es_proxy_command_windows()  # Install aws-es-proxy on Windows
    else:
        print(f"Unsupported OS: {os_name}")
        return

    print("Installation of aws-es-proxy completed!")


def install_brew():
    """
    Check if Homebrew is installed on macOS. If not, install Homebrew.
    Homebrew is a package manager for macOS that makes it easy to install software.
    """
    try:
        subprocess.check_call(["brew", "--version"])
        print("Homebrew is already installed.")
    except subprocess.CalledProcessError:
        print("Homebrew not found. Installing Homebrew...")
        try:
            subprocess.check_call(
                ['/bin/bash', '-c',
                 '"$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"']
            )
            print("Homebrew installed successfully.")
        except subprocess.CalledProcessError as e:
            print(f"Failed to install Homebrew: {e}")


def install_go_mac():
    """
    Install Go programming language on macOS using Homebrew.
    Go is required to install aws-es-proxy.
    """
    try:
        subprocess.check_call(["brew", "install", "go"])
        print("Go installed successfully on macOS.")
    except subprocess.CalledProcessError as e:
        print(f"Failed to install Go on macOS: {e}")


def install_go_linux():
    """
    Install Go programming language on Linux using apt package manager.
    Go is required to install aws-es-proxy.
    """
    try:
        subprocess.check_call(["sudo", "apt", "update"])
        subprocess.check_call(["sudo", "apt", "install", "-y", "golang-go"])
        print("Go installed successfully on Linux.")
    except subprocess.CalledProcessError as e:
        print(f"Failed to install Go on Linux: {e}")


def install_go_windows():
    """
    Install Go programming language on Windows.
    This function downloads the Go installer and runs it.
    Go is required to install aws-es-proxy.
    """
    go_url = "https://golang.org/dl/go1.16.6.windows-amd64.msi"  # URL for the Go installer MSI file
    msi_path = "go-installer.msi"  # Local path to save the downloaded MSI file

    try:
        subprocess.check_call(["curl", "-o", msi_path, go_url])
        subprocess.check_call(["msiexec", "/i", msi_path, "/quiet", "/norestart"])
        print("Go installed successfully on Windows.")
    except subprocess.CalledProcessError as e:
        print(f"Failed to install Go on Windows: {e}")
    finally:
        if os.path.exists(msi_path):
            os.remove(msi_path)  # Clean up the MSI installer file after installation


def set_environment_variable_mac_linux():
    """
    Set the environment variables for Go and aws-es-proxy on macOS and Linux.
    This function updates the PATH environment variable to include the Go bin directory.
    """
    go_path = os.path.expanduser("~/go/bin")

    shell_config_file = os.path.expanduser("~/.bashrc")  # Default shell config file for bash
    if 'zsh' in os.environ.get('SHELL', ''):
        shell_config_file = os.path.expanduser("~/.zshrc")  # zsh shell config file

    export_command = f'\nexport PATH=$PATH:{go_path}\n'

    try:
        with open(shell_config_file, 'a') as f:
            f.write(export_command)
        print(f"Environment variables set successfully on macOS/Linux. Added {go_path} to PATH.")
        print(f"Please run 'source {shell_config_file}' or open a new terminal session to apply the changes.")
    except Exception as e:
        print(f"Failed to set environment variables on macOS/Linux: {e}")


def set_environment_variable_windows():
    """
    Set the environment variables for Go and aws-es-proxy on Windows.
    This function updates the PATH environment variable to include the Go bin directory.
    """
    go_path = os.path.join(os.environ['USERPROFILE'], 'go', 'bin')

    try:
        subprocess.check_call(f'setx PATH "%PATH%;{go_path}"', shell=True)
        print(f"Environment variables set successfully on Windows. Added {go_path} to PATH.")
        print("Please restart your command prompt or open a new one to apply the changes.")
    except subprocess.CalledProcessError as e:
        print(f"Failed to set environment variables on Windows: {e}")


def install_aws_es_proxy_command():
    """
    Install aws-es-proxy using Go's package management.
    This command works for both macOS and Linux.
    """
    try:
        subprocess.check_call(["go", "install", "github.com/abutaha/aws-es-proxy@latest"])
        print("aws-es-proxy installed successfully.")
    except subprocess.CalledProcessError as e:
        print(f"Failed to install aws-es-proxy: {e}")


def install_aws_es_proxy_command_windows():
    """
    Install aws-es-proxy on Windows using Go's package management.
    """
    try:
        subprocess.check_call(["go", "install", "github.com/abutaha/aws-es-proxy@latest"], shell=True)
        print("aws-es-proxy installed successfully.")
    except subprocess.CalledProcessError as e:
        print(f"Failed to install aws-es-proxy: {e}")


if __name__ == "__main__":
    install_aws_es_proxy()



"""
앱에서 쓰일 에러를 정의합니다.
"""


from sawsi.shared.error_util import AppError


err_n = 1

def auto(err_type:str):
    global err_n
    # 자동으로 에러 카운트를 증가시킴
    err_type = err_type.capitalize()
    err_s = f'{err_type}{str(err_n).rjust(4, "0")}'
    err_n += 1
    return err_s


source_ip_not_in_allow_ips = AppError(auto('S'), 'source_ip_not_in_allow_ips')
source_ip_not_match = AppError(auto('S'), 'source_ip_not_match')
permission_denied = AppError(auto('S'), 'permission_denied')
no_session = AppError(auto('S'), 'no_session')


def print_all_errors():
    items = list(globals().items())
    for name, obj in items:
        # AppError의 인스턴스인지 확인
        if isinstance(obj, AppError):
            print(f'Code: {obj.code}, Message: {obj.message}')


if __name__ == '__main__':
    # 현재 모듈의 모든 속성을 가져옴
    print_all_errors()




from sawsi.aws import shared
import json


class Bedrock:
    def __init__(self, boto3_session, region=shared.DEFAULT_REGION):
        self.client = boto3_session.client('bedrock-runtime', region_name=region)
        self.region = region

    def invoke_model(self, model_id, body, content_type='application/json', accept='application/json'):
        response = self.client.invoke_model(
            modelId=model_id,
            body=json.dumps(body) if isinstance(body, dict) else body,
            contentType=content_type,
            accept=accept,
        )
        return json.loads(response['body'].read())

    def converse(self, model_id, messages, system=None, inference_config=None, tool_config=None):
        params = {
            'modelId': model_id,
            'messages': messages,
        }
        if system:
            params['system'] = system
        if inference_config:
            params['inferenceConfig'] = inference_config
        if tool_config:
            params['toolConfig'] = tool_config
        response = self.client.converse(**params)
        return response

    def converse_stream(self, model_id, messages, system=None, inference_config=None, tool_config=None):
        params = {
            'modelId': model_id,
            'messages': messages,
        }
        if system:
            params['system'] = system
        if inference_config:
            params['inferenceConfig'] = inference_config
        if tool_config:
            params['toolConfig'] = tool_config
        response = self.client.converse_stream(**params)
        return response



from sawsi.aws import shared
from sawsi.aws.s3_vector import wrapper
from typing import Optional, Literal
import json


class S3VectorAPI:
    """
    Amazon S3 Vectors 를 활용하는 벡터 저장/검색 API
    """
    def __init__(self, vector_bucket_name, credentials=None, region=shared.DEFAULT_REGION):
        """
        :param vector_bucket_name: 벡터 버킷 이름
        :param credentials: {
            "aws_access_key_id": "str",
            "aws_secret_access_key": "str",
            "region_name": "str",
            "profile_name": "str",
        }
        """
        self.boto3_session = shared.get_boto_session(credentials)
        self.vector_bucket_name = vector_bucket_name
        self.s3_vector = wrapper.S3Vector(self.boto3_session, region=region)

    def init_vector_bucket(self):
        """
        벡터 버킷 생성
        """
        try:
            return self.s3_vector.create_vector_bucket(self.vector_bucket_name)
        except Exception as ex:
            print(ex)

    def delete_vector_bucket(self):
        return self.s3_vector.delete_vector_bucket(self.vector_bucket_name)

    def get_vector_bucket(self):
        return self.s3_vector.get_vector_bucket(self.vector_bucket_name)

    def create_index(self, index_name, dimension, distance_metric='cosine', metadata_schema=None):
        """
        벡터 인덱스 생성
        :param index_name: 인덱스 이름
        :param dimension: 벡터 차원 수
        :param distance_metric: 거리 메트릭 ('cosine', 'euclidean', 'dotproduct')
        :param metadata_schema: 메타데이터 스키마 (dict)
        """
        return self.s3_vector.create_vector_index(
            self.vector_bucket_name, index_name, dimension, distance_metric, metadata_schema
        )

    def delete_index(self, index_name):
        return self.s3_vector.delete_vector_index(self.vector_bucket_name, index_name)

    def list_indexes(self, max_results=100, next_token=None):
        return self.s3_vector.list_vector_indexes(self.vector_bucket_name, max_results, next_token)

    def get_index(self, index_name):
        return self.s3_vector.get_vector_index(self.vector_bucket_name, index_name)

    def put_vectors(self, index_name, vectors):
        """
        벡터 저장
        :param index_name: 인덱스 이름
        :param vectors: [{'key': 'id1', 'data': {'float32': [0.1, 0.2, ...]}, 'metadata': {...}}, ...]
        """
        return self.s3_vector.put_vectors(self.vector_bucket_name, index_name, vectors)

    def put_vector(self, index_name, key, vector, metadata=None):
        """
        단일 벡터 저장
        :param index_name: 인덱스 이름
        :param key: 벡터 ID
        :param vector: 벡터 데이터 (list[float])
        :param metadata: 메타데이터 (dict)
        """
        item = {
            'key': key,
            'data': {'float32': vector},
        }
        if metadata:
            item['metadata'] = metadata
        return self.s3_vector.put_vectors(self.vector_bucket_name, index_name, [item])

    def get_vectors(self, index_name, keys):
        """
        벡터 조회
        :param index_name: 인덱스 이름
        :param keys: 벡터 ID 리스트
        """
        return self.s3_vector.get_vectors(self.vector_bucket_name, index_name, keys)

    def delete_vectors(self, index_name, keys):
        return self.s3_vector.delete_vectors(self.vector_bucket_name, index_name, keys)

    def query(self, index_name, query_vector, top_k=10, filter_expression=None):
        """
        벡터 유사도 검색
        :param index_name: 인덱스 이름
        :param query_vector: 쿼리 벡터 (list[float])
        :param top_k: 반환할 결과 수
        :param filter_expression: 메타데이터 필터
        :return: 유사한 벡터 목록
        """
        return self.s3_vector.query_vectors(
            self.vector_bucket_name, index_name, query_vector, top_k, filter_expression
        )

    def put_vectors_batch(self, index_name, keys, vectors, metadata_list=None, batch_size=100):
        """
        벡터 배치 저장
        :param index_name: 인덱스 이름
        :param keys: 벡터 ID 리스트
        :param vectors: 벡터 데이터 리스트 (list[list[float]])
        :param metadata_list: 메타데이터 리스트 (list[dict])
        :param batch_size: 배치 크기
        """
        items = []
        for i, (key, vector) in enumerate(zip(keys, vectors)):
            item = {
                'key': key,
                'data': {'float32': vector},
            }
            if metadata_list and i < len(metadata_list):
                item['metadata'] = metadata_list[i]
            items.append(item)

        results = []
        for i in range(0, len(items), batch_size):
            batch = items[i:i + batch_size]
            result = self.s3_vector.put_vectors(self.vector_bucket_name, index_name, batch)
            results.append(result)
        return results


if __name__ == '__main__':
    api = S3VectorAPI('')



from sawsi.aws import shared
from sawsi.aws.bedrock import wrapper
from typing import Optional


class BedrockAPI:
    """
    AWS Bedrock 런타임 API
    """
    def __init__(self, credentials=None, region=shared.DEFAULT_REGION):
        """
        :param credentials: {
            "aws_access_key_id": "str",
            "aws_secret_access_key": "str",
            "region_name": "str",
            "profile_name": "str",
        }
        """
        self.boto3_session = shared.get_boto_session(credentials)
        self.bedrock = wrapper.Bedrock(self.boto3_session, region=region)

    def invoke_model(self, model_id, body, content_type='application/json', accept='application/json'):
        """
        모델 직접 호출 (로우레벨)
        """
        return self.bedrock.invoke_model(model_id, body, content_type, accept)

    def converse(self, model_id, messages, system=None, max_tokens=None, temperature=None, top_p=None, tool_config=None):
        """
        Converse API 를 통한 모델 호출
        :param model_id: 모델 ID (예: 'anthropic.claude-sonnet-4-20250514')
        :param messages: [{'role': 'user', 'content': [{'text': '...'}]}]
        :param system: 시스템 프롬프트 (str 또는 [{'text': '...'}])
        :param max_tokens: 최대 토큰 수
        :param temperature: 온도
        :param top_p: top_p
        :param tool_config: 도구 설정
        """
        system_param = None
        if system:
            if isinstance(system, str):
                system_param = [{'text': system}]
            else:
                system_param = system

        inference_config = {}
        if max_tokens:
            inference_config['maxTokens'] = max_tokens
        if temperature is not None:
            inference_config['temperature'] = temperature
        if top_p is not None:
            inference_config['topP'] = top_p

        return self.bedrock.converse(
            model_id, messages,
            system=system_param,
            inference_config=inference_config if inference_config else None,
            tool_config=tool_config,
        )

    def converse_stream(self, model_id, messages, system=None, max_tokens=None, temperature=None, top_p=None, tool_config=None):
        """
        Converse API 스트리밍 호출
        """
        system_param = None
        if system:
            if isinstance(system, str):
                system_param = [{'text': system}]
            else:
                system_param = system

        inference_config = {}
        if max_tokens:
            inference_config['maxTokens'] = max_tokens
        if temperature is not None:
            inference_config['temperature'] = temperature
        if top_p is not None:
            inference_config['topP'] = top_p

        return self.bedrock.converse_stream(
            model_id, messages,
            system=system_param,
            inference_config=inference_config if inference_config else None,
            tool_config=tool_config,
        )

    def chat(self, model_id, user_message, system=None, max_tokens=1024, temperature=None):
        """
        간단한 채팅 메시지 전송
        :param model_id: 모델 ID
        :param user_message: 사용자 메시지 (str)
        :param system: 시스템 프롬프트 (str)
        :param max_tokens: 최대 토큰 수
        :param temperature: 온도
        :return: 응답 텍스트 (str)
        """
        messages = [
            {'role': 'user', 'content': [{'text': user_message}]}
        ]
        response = self.converse(model_id, messages, system=system, max_tokens=max_tokens, temperature=temperature)
        return response['output']['message']['content'][0]['text']

    def get_embeddings(self, model_id, input_text, dimensions=None, normalize=True):
        """
        임베딩 벡터 생성 (Amazon Titan Embeddings 등)
        :param model_id: 임베딩 모델 ID (예: 'amazon.titan-embed-text-v2:0')
        :param input_text: 입력 텍스트
        :param dimensions: 벡터 차원 수
        :param normalize: 정규화 여부
        :return: 임베딩 벡터 (list[float])
        """
        body = {
            'inputText': input_text,
        }
        if dimensions:
            body['dimensions'] = dimensions
        if normalize is not None:
            body['normalize'] = normalize
        response = self.invoke_model(model_id, body)
        return response.get('embedding', response)

    def generate_stream_text(self, model_id, messages, system=None, max_tokens=1024, temperature=None):
        """
        스트리밍 텍스트 생성 제너레이터
        :return: 텍스트 청크를 yield
        """
        response = self.converse_stream(model_id, messages, system=system, max_tokens=max_tokens, temperature=temperature)
        stream = response.get('stream', [])
        for event in stream:
            if 'contentBlockDelta' in event:
                delta = event['contentBlockDelta'].get('delta', {})
                if 'text' in delta:
                    yield delta['text']


if __name__ == '__main__':
    api = BedrockAPI()



from sawsi.aws import shared
import json


class S3Vector:
    def __init__(self, boto3_session, region=shared.DEFAULT_REGION):
        self.client = boto3_session.client('s3vectors', region_name=region)
        self.region = region

    def create_vector_bucket(self, vector_bucket_name, encryption_type='aws:s3vectors'):
        """
        벡터 버킷 생성
        """
        params = {
            'vectorBucketName': vector_bucket_name,
            'encryptionConfiguration': {
                'sseType': encryption_type,
            },
        }
        response = self.client.create_vector_bucket(**params)
        return response

    def delete_vector_bucket(self, vector_bucket_name):
        response = self.client.delete_vector_bucket(vectorBucketName=vector_bucket_name)
        return response

    def list_vector_buckets(self, max_results=100, next_token=None):
        params = {'maxResults': max_results}
        if next_token:
            params['nextToken'] = next_token
        response = self.client.list_vector_buckets(**params)
        return response

    def get_vector_bucket(self, vector_bucket_name):
        response = self.client.get_vector_bucket(vectorBucketName=vector_bucket_name)
        return response

    def create_vector_index(self, vector_bucket_name, vector_index_name, dimension, distance_metric='cosine', metadata_schema=None):
        """
        벡터 인덱스 생성
        :param vector_bucket_name: 벡터 버킷 이름
        :param vector_index_name: 벡터 인덱스 이름
        :param dimension: 벡터 차원 수
        :param distance_metric: 거리 메트릭 ('cosine', 'euclidean', 'dotproduct')
        :param metadata_schema: 메타데이터 스키마 (dict)
        """
        params = {
            'vectorBucketName': vector_bucket_name,
            'vectorIndexName': vector_index_name,
            'dimension': dimension,
            'distanceMetric': distance_metric,
        }
        if metadata_schema:
            params['metadataConfiguration'] = {
                'metadataSchema': json.dumps(metadata_schema),
            }
        response = self.client.create_vector_index(**params)
        return response

    def delete_vector_index(self, vector_bucket_name, vector_index_name):
        response = self.client.delete_vector_index(
            vectorBucketName=vector_bucket_name,
            vectorIndexName=vector_index_name,
        )
        return response

    def list_vector_indexes(self, vector_bucket_name, max_results=100, next_token=None):
        params = {
            'vectorBucketName': vector_bucket_name,
            'maxResults': max_results,
        }
        if next_token:
            params['nextToken'] = next_token
        response = self.client.list_vector_indexes(**params)
        return response

    def get_vector_index(self, vector_bucket_name, vector_index_name):
        response = self.client.get_vector_index(
            vectorBucketName=vector_bucket_name,
            vectorIndexName=vector_index_name,
        )
        return response

    def put_vectors(self, vector_bucket_name, vector_index_name, vectors):
        """
        벡터 저장
        :param vectors: [{'key': 'id1', 'data': {'float32': [0.1, 0.2, ...]}, 'metadata': {...}}, ...]
        """
        response = self.client.put_vectors(
            vectorBucketName=vector_bucket_name,
            vectorIndexName=vector_index_name,
            vectors=vectors,
        )
        return response

    def get_vectors(self, vector_bucket_name, vector_index_name, keys):
        """
        벡터 조회
        :param keys: ['id1', 'id2', ...]
        """
        response = self.client.get_vectors(
            vectorBucketName=vector_bucket_name,
            vectorIndexName=vector_index_name,
            keys=keys,
        )
        return response

    def delete_vectors(self, vector_bucket_name, vector_index_name, keys):
        response = self.client.delete_vectors(
            vectorBucketName=vector_bucket_name,
            vectorIndexName=vector_index_name,
            keys=keys,
        )
        return response

    def query_vectors(self, vector_bucket_name, vector_index_name, query_vector, top_k=10, filter_expression=None):
        """
        벡터 유사도 검색
        :param query_vector: 쿼리 벡터 (list[float])
        :param top_k: 반환할 결과 수
        :param filter_expression: 메타데이터 필터
        """
        params = {
            'vectorBucketName': vector_bucket_name,
            'vectorIndexName': vector_index_name,
            'queryVector': {'float32': query_vector},
            'topK': top_k,
        }
        if filter_expression:
            params['filter'] = filter_expression
        response = self.client.query_vectors(**params)
        return response



"""
This is sample code to tell how to use sawsi framework.
You could delete this file after know how to use this framework.
"""
from {{app}}.model.sample_user import User
from pydantic import validate_call
from sawsi.handler.controller_decorator import controller

@controller
@validate_call
def login(email:str, password:str):
    return {
        'status': 'succeed'
    }


@controller
@validate_call  # session_id 에 int 등이 등어가면 에러가 레이즈됩니다.
def get_me(session_id:str):
    # Get Data Model from DAO
    user_id = some_function_session_id_to_user_id(session_id)
    user = User.get(user_id)
    user_view_model = user.view()
    return {
        'user': user_view_model
    }


@controller
def some_function_session_id_to_user_id(session_id: str)->str:
    # DEMO FUNCTION
    return 'USER_ID'


from sawsi.aws import shared
from sawsi.aws.sqs import wrapper


class SQSAPI:
    """
    큐 API
    """
    def __init__(self, queue_url, credentials=None, region=shared.DEFAULT_REGION):
        """
        :param credentials: {
            "aws_access_key_id": "str",
            "aws_secret_access_key": "str",
            "region_name": "str",
            "profile_name": "str",
        }
        """
        self.boto3_session = shared.get_boto_session(credentials)
        self.region_name = region
        self.sqs = wrapper.SQS(self.boto3_session, region)
        self.queue_url = queue_url

    def send_message(self, message_body):
        response = self.sqs.send_message(self.queue_url, message_body)
        return response


if __name__ == '__main__':
    pass



"""
빌드 준비에 필요한 코드 호출
샘플입니다. 적절히 변경해서 사용해주세요.
"""


import api
import config
import traceback
from {{app}}.model.sample_user import Address, User
from sawsi.model.base import sync_all_models


def run():
    config.build = True
    # DB 초기화 등 진행.

    sync_all_models([
        "{{app}}.model"
    ])
    api.locking.init_table()
    api.firehose_log.init()
    api.s3_public.init_s3_bucket()
    # 초기화 메서드를 제공하지 않는 리소스는, AWS Console 에서 직접 생성 요망

    if config.env == 'dev':
        import {{app}}.doc_maker
        {{app}}.doc_maker.make_api_doc()


if __name__ == '__main__':
    try:
        run()
    except Exception as e:
        raise Exception(f'Exception:{e}, \ntraceback:{traceback.format_exc()}')
        # 에러를 다시 발생시켜 CodeBuild가 실패 상태를 감지할 수 있도록 합니다.



from sawsi.aws import shared
from sawsi.aws.bedrock.api import BedrockAPI
from sawsi.aws.s3_vector.api import S3VectorAPI
from typing import Optional
from secrets import token_urlsafe


DEFAULT_EMBED_MODEL = 'amazon.titan-embed-text-v2:0'
DEFAULT_DIMENSION = 1024


class VectorKBAPI:
    """
    텍스트 기반 벡터 지식베이스 API
    Bedrock 임베딩 + S3 Vectors 를 조합하여
    텍스트만 넣으면 자동으로 임베딩 → 저장 → 검색까지 처리합니다.

    사용 예:
        kb = VectorKBAPI('my-kb-bucket', 'my-index')
        kb.init()

        # 텍스트 저장
        kb.add('doc-1', '서울은 대한민국의 수도입니다.', metadata={'source': 'wiki'})

        # 검색
        results = kb.search('한국의 수도는?', top_k=3)

        # 배치 저장
        kb.add_batch([
            {'key': 'doc-2', 'text': '부산은 항구도시입니다.'},
            {'key': 'doc-3', 'text': '제주도는 섬입니다.'},
        ])
    """
    def __init__(
        self,
        vector_bucket_name,
        index_name,
        embed_model_id=DEFAULT_EMBED_MODEL,
        dimension=DEFAULT_DIMENSION,
        distance_metric='cosine',
        credentials=None,
        region=shared.DEFAULT_REGION,
    ):
        """
        :param vector_bucket_name: S3 Vectors 버킷 이름
        :param index_name: 벡터 인덱스 이름
        :param embed_model_id: Bedrock 임베딩 모델 ID (기본: Titan Embed Text v2)
        :param dimension: 벡터 차원 수 (기본: 1024)
        :param distance_metric: 거리 메트릭 ('cosine', 'euclidean', 'dotproduct')
        :param credentials: AWS 자격 증명
        :param region: AWS 리전
        """
        self.index_name = index_name
        self.embed_model_id = embed_model_id
        self.dimension = dimension
        self.distance_metric = distance_metric
        self.bedrock = BedrockAPI(credentials=credentials, region=region)
        self.vector_store = S3VectorAPI(vector_bucket_name, credentials=credentials, region=region)

    def init(self, metadata_schema=None):
        """
        벡터 버킷 및 인덱스를 초기화합니다. (최초 1회)
        :param metadata_schema: 메타데이터 스키마 (dict), 예: {"type": "object", "properties": {"source": {"type": "string"}}}
        """
        self.vector_store.init_vector_bucket()
        try:
            self.vector_store.create_index(
                self.index_name, self.dimension, self.distance_metric, metadata_schema
            )
        except Exception as ex:
            print(ex)

    def _embed(self, text):
        """텍스트를 벡터로 변환"""
        return self.bedrock.get_embeddings(
            self.embed_model_id, text, dimensions=self.dimension
        )

    def add(self, key, text, metadata=None):
        """
        텍스트를 임베딩하여 벡터 저장소에 저장합니다.
        :param key: 문서 ID
        :param text: 저장할 텍스트
        :param metadata: 메타데이터 (dict)
        :return: 저장 결과
        """
        vector = self._embed(text)
        meta = metadata or {}
        meta['text'] = text
        return self.vector_store.put_vector(self.index_name, key, vector, metadata=meta)

    def add_auto_key(self, text, metadata=None):
        """
        키를 자동 생성하여 텍스트를 저장합니다.
        :param text: 저장할 텍스트
        :param metadata: 메타데이터 (dict)
        :return: (생성된 key, 저장 결과)
        """
        key = token_urlsafe(16)
        result = self.add(key, text, metadata)
        return key, result

    def add_batch(self, items, batch_size=10):
        """
        여러 텍스트를 배치로 임베딩하여 저장합니다.
        :param items: [{'key': 'doc-1', 'text': '...', 'metadata': {...}}, ...]
                      metadata 는 선택사항
        :param batch_size: 한 번에 저장할 벡터 수
        :return: 저장 결과 리스트
        """
        keys = []
        vectors = []
        metadata_list = []

        for item in items:
            key = item.get('key', token_urlsafe(16))
            text = item['text']
            meta = item.get('metadata', {})
            meta['text'] = text

            keys.append(key)
            vectors.append(self._embed(text))
            metadata_list.append(meta)

        return self.vector_store.put_vectors_batch(
            self.index_name, keys, vectors, metadata_list, batch_size
        )

    def search(self, query_text, top_k=5, filter_expression=None):
        """
        텍스트로 유사도 검색을 수행합니다.
        :param query_text: 검색 쿼리 텍스트
        :param top_k: 반환할 결과 수
        :param filter_expression: 메타데이터 필터
        :return: 유사한 벡터 목록
        """
        query_vector = self._embed(query_text)
        return self.vector_store.query(
            self.index_name, query_vector, top_k, filter_expression
        )

    def get(self, keys):
        """
        키로 벡터를 조회합니다.
        :param keys: 벡터 ID 리스트
        """
        if isinstance(keys, str):
            keys = [keys]
        return self.vector_store.get_vectors(self.index_name, keys)

    def delete(self, keys):
        """
        키로 벡터를 삭제합니다.
        :param keys: 벡터 ID 리스트
        """
        if isinstance(keys, str):
            keys = [keys]
        return self.vector_store.delete_vectors(self.index_name, keys)


if __name__ == '__main__':
    kb = VectorKBAPI('my-kb-bucket', 'my-index')
    kb.add('1', 'SOME TEXT')
    kb.add('2', 'HELLO')
    r = kb.search('SOME')
    print(r)
